package com.yunzhi.niucai.common;

import java.util.ArrayList;
import java.util.List;

/**
 * 业务相关工具类
 */
public class BizUtils {
    /**
     * 计算竞彩过关总数
     * 计算方式如下, 比如购买 5 场  4串5
     * 那么从第一场开始与剩下的4场进行组合，求可能的组合数
     * 接着第二场与剩下的3场进行组合，求可能的组合数
     * 依次下去，直到剩余的场次不能组合4串1
     * 示例 5场的投注注数为 [1, 2, 3, 4, 2]
     * 第一场则有 4种组合可能，即 剩余的 4场挑 3场与第一场组合, C3-4 就是 4
     *     [1, 2, 3, 4] = 1 * 2 * 3 * 4 = 24
     *     [1, 2, 3, 2] = 1 * 2 * 3 * 2 = 12
     *     [1, 2, 4, 2] = 1 * 2 * 4 * 2 = 16
     *     [1, 3, 4, 2] = 1 * 3 * 4 * 2 = 24
     *   第一场结果就是 24 + 12 + 16 + 24 = 76
     * 第二场有 1 种可能, 即 只能把剩余的 3场全部拿来组合
     *     [2, 3, 4, 2] = 2 * 3 * 4 * 2 = 48
     * 剩余的都不能再组合 4串1
     * 因此总过关数是  (76 + 48) * 5  这个 5 来源 4串5
     * @param unitNum
     * @param amount
     * @param noteNumList
     * @return
     */
    public static int computeSportNoteNum(int unitNum, int amount, List<Integer> noteNumList) {
        if (null == noteNumList || noteNumList.size() == 0) {
            return 0;
        }

        List<List<Integer>> result = new ArrayList<>();
        int times = 0;
        int cursor = 0;

        // 比如 5 场 投注 4串5, 只可能循环2次, 因为后面的都不能组合为 4串1
        int max = noteNumList.size() - (unitNum - times);

        // 遍历每场的可能
        for(int i = cursor; i <= max; i++) {
            List<Integer> last = new ArrayList<>();
            last.add(noteNumList.get(i));
            recursion(noteNumList, unitNum, times, cursor, last, result);
        }

        // 遍历结果, 外循环是场次的结果，需要相加
        // 内循环是每场的组合可能, 需要相乘
        int sum = 0;
        for(List<Integer> noteNum: result) {
            int multi = 1;
            for(Integer n: noteNum) {
                multi *= n;
            }
            sum += multi;
        }

        return sum * amount;
    }

    /**
     * 递归遍历每种排列组合方式
     * @param noteNumList
     * @param unitNum 过关场数 比如 4串1 , 4 就是场数
     * @param times 当前场数 0 开始
     * @param cursor noteNumList 的 当前位置
     * @param last  上一轮结果
     * @param out 通过 ref 方式输出结果
     */
    private static void recursion(List<Integer> noteNumList, int unitNum, int times, int cursor, List<Integer> last, List<List<Integer>> out) {
        int max = noteNumList.size() - (unitNum - times);

        for (int i = cursor; i <= max; i ++) {
            List<Integer> current = last;
            current.add(noteNumList.get(i));

            if (times == unitNum - 1) {
                out.add(current);
            } else {
                // 每一轮只能取后面的一个位置数据
                recursion(noteNumList, unitNum, times + 1, i + 1, current, out);
            }
        }
    }
}
