package com.yunzhi.niucai.service.impl;

import com.yunzhi.niucai.common.BizUtils;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.*;
import com.yunzhi.niucai.enums.SportStatusEnum;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaBettingPlanMapper;
import com.yunzhi.niucai.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 投注方案 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-08
 */
@Service
public class TaBettingPlanServiceImpl extends ServiceImpl<TaBettingPlanMapper, TaBettingPlan> implements ITaBettingPlanService {

    @Autowired
    ITdLotteryService iTdLotteryService;

    @Autowired
    ITdSportPassService iTdSportPassService;

    @Autowired
    ITaMatchService iTaMatchService;

    @Autowired
    ITaMatchWayService iTaMatchWayService;

    @Autowired
    ITaMatchOddsService iTaMatchOddsService;

    @Autowired
    ITaShopService iTaShopService;

    @Override
    public boolean betting(TaBettingPlan taBettingPlan, TaCustomer customer) throws Exception {
        //
        validateBetting(taBettingPlan);

        taBettingPlan.setCustomerId(customer.getCustomerId());
        taBettingPlan.setStatus(StatusEnum.READY.getCode());

        // 入库

        // 我的

        // 下单
        return true;
    }

    private void validateShop(TaBettingPlan taBettingPlan) throws Exception {
        if (null == taBettingPlan.getShopId()) {
            throw new Exception("请选择投注店铺");
        }

        TaShop taShop = iTaShopService.getById(taBettingPlan.getShopId());
        if (null == taShop) {
            throw new Exception("投注店铺不存在");
        }

        // 店铺状态不对
        if (!StatusEnum.NORMAL.getCode().equals(taShop.getStatus())) {
            throw new Exception("投注店铺状态异常, 请更换");
        }

        taBettingPlan.setShopName(taShop.getName());
    }

    private void validateBetting(TaBettingPlan taBettingPlan) throws Exception {
        if (null == taBettingPlan.getLotteryId()) {
            throw new Exception("投注彩种不能未空");
        }

        TdLottery tdLottery = iTdLotteryService.getById(taBettingPlan.getLotteryId());
        if (null == tdLottery) {
            throw new Exception("投注彩种不合法");
        }
        taBettingPlan.setLotteryName(tdLottery.getName());
        taBettingPlan.setIsDigit(tdLottery.getIsSport());
        taBettingPlan.setIsDigit(tdLottery.getIsDigit());

        // 校验彩种
        validateLottery(taBettingPlan, tdLottery);

        //
        validateShop(taBettingPlan);

        // 校验明细
        if (null == taBettingPlan.getItemList()) {
            throw new Exception("请设置投注内容");
        }

        // 如果是体彩
        if (tdLottery.getIsSport()) {
            if (StringUtils.isEmpty(taBettingPlan.getPassCode())) {
                throw new Exception("过关方式不能为空");
            }

            TdSportPass tdSportPass = iTdSportPassService.getById(taBettingPlan.getPassCode());
            if (null == tdSportPass) {
                throw new Exception("过关方式不合法");
            }
            taBettingPlan.setPassName(tdSportPass.getName());

            List<TaBettingPlanItem> itemList = taBettingPlan.getItemList();
            if (null == itemList || tdSportPass.getUnitNum() > itemList.size()) {
                throw new Exception("投注比赛场次数量不正确");
            }

            List<Integer> noteNumList = new ArrayList<>();
            for (TaBettingPlanItem item : taBettingPlan.getItemList()) {
                validateSport(taBettingPlan, item, tdLottery);
                noteNumList.add(item.getDetailList().size());
            }

            // 校验注数
            int totalNums = BizUtils.computeSportNoteNum(tdSportPass.getUnitNum(), tdSportPass.getAmount(), noteNumList);
            if (null == taBettingPlan.getNotesNum() || !taBettingPlan.getNotesNum().equals(totalNums)) {
                throw new Exception("投注总注数计算不正确, 应该为: " + totalNums);
            }

            // 总金额
            int totalPrice = totalNums * tdLottery.getUnitPrice();
            if (null == taBettingPlan.getTotalPrice() || !taBettingPlan.getTotalPrice().equals(totalPrice)) {
                throw new Exception("投注总金额计算不正确");
            }

            if (totalPrice > tdLottery.getMaxPrice()) {
                throw new Exception("投注总金额不能超过 " + String.format("%.2f", tdLottery.getMaxPrice() / 100 ) + "元");
            }
        }

        // 如果是数字彩
        if (tdLottery.getIsDigit()) {
            Integer totalCharges = 0;
            Integer totalNoteNum = 0;
            for (TaBettingPlanItem item : taBettingPlan.getItemList()) {
                validateDigit(taBettingPlan, item, tdLottery);
                totalCharges += item.getCharges();
                totalNoteNum += item.getAmount();
            }

            if (null == taBettingPlan.getTotalPrice()) {
                taBettingPlan.setTotalPrice(totalCharges);
            }
            if (null == taBettingPlan.getNotesNum()) {
                taBettingPlan.setNotesNum(totalNoteNum);
            }

            if (!taBettingPlan.getTotalPrice().equals(totalCharges)) {
                throw new Exception("投注总金额计算不正确");
            }

            // 不能超过限额
            Integer maxPrice = tdLottery.getMaxPrice();
            if (tdLottery.getIsAdd()) {
                maxPrice = tdLottery.getMaxAddPrice();
            }
            if (maxPrice < totalCharges) {
                throw new Exception("投注总金额不能超过 " + String.format("%.2f", maxPrice / 100 ) + "元");
            }
        }

        return;
    }

    private void validateLottery(TaBettingPlan taBettingPlan, TdLottery tdLottery) throws Exception {
        if (!StatusEnum.NORMAL.getCode().equals(tdLottery.getStatus())) {
            throw new Exception("当前投注彩种被停用");
        }

        if (null == taBettingPlan.getTimes() || tdLottery.getMaxTimes() < taBettingPlan.getTimes()) {
            throw new Exception("当前投注倍数太大(小), 不能超过" + tdLottery.getMaxTimes());
        }

        // todo 投注期数

        // todo 校验停售时间

    }

    private void validateSport(TaBettingPlan taBettingPlan,
                                       TaBettingPlanItem taBettingPlanItem,
                                       TdLottery tdLottery) throws Exception {
        // 获取比赛信息
        TaMatch taMatch = iTaMatchService.getById(taBettingPlanItem.getMatchId());
        if (null == taMatch) {
            throw new Exception("未投注比赛或者比赛不存在");
        }
        taBettingPlanItem.setLeagueId(taMatch.getLeagueId());
        taBettingPlanItem.setLeagueName(taMatch.getLeagueName());
        taBettingPlanItem.setMatchWeek(taMatch.getMatchWeek());
        taBettingPlanItem.setMatchDate(taMatch.getMatchDate());
        taBettingPlanItem.setHomeTeamId(taMatch.getHomeTeamId());
        taBettingPlanItem.setHomeTeamName(taMatch.getHomeTeamName());
        taBettingPlanItem.setAwayTeamId(taMatch.getAwayTeamId());
        taBettingPlanItem.setAwayTeamName(taMatch.getAwayTeamName());

        // 如果当前比赛不是在售
        if (!SportStatusEnum.SALE.getCode().equals(taMatch.getStatus())) {
            throw new Exception("投注比赛非在售状态");
        }

        // 投注内容
        if (null == taBettingPlanItem.getDetailList() || taBettingPlanItem.getDetailList().size() == 0) {
            throw new Exception("未找到比赛的投注内容");
        }

        // 当前比赛玩法
        Map<String, TaMatchWay> taMatchWayMap = iTaMatchWayService.getGroupedByWay(taMatch.getMatchId());
        if (null == taMatchWayMap) {
            throw new Exception("未找到投注比赛玩法设置");
        }

//        // 当前比赛赔率
//        Map<String, TaMatchOdds> taMatchOddsMap = iTaMatchOddsService.getGroupedByRule(taMatch.getMatchId());
//        if (null == taMatchOddsMap) {
//            throw new Exception("未找到投注比赛赔率设置");
//        }

        // 单关-过关验证
        boolean isSinglePass = iTdSportPassService.getSinglePassCode().equals(taBettingPlan.getPassCode());

        // 玩法等验证
        for (TaBettingPlanSportDetail detail: taBettingPlanItem.getDetailList()) {
            detail.setMatchId(taMatch.getMatchId());

            if (StringUtils.isEmpty(detail.getOdds())) {
                throw new Exception("未找到投注比赛明细赔率设置");
            }

            TaMatchWay taMatchWay = taMatchWayMap.get(detail.getWayCode());
            if (null == taMatchWay) {
                throw new Exception("未找到投注比赛明细玩法设置");
            }

            if (null == taMatchWay.getIsOn() || !taMatchWay.getIsOn()) {
                throw new Exception("投注比赛明细未开启购彩玩法");
            }

            if (isSinglePass) {
                if (null == taMatchWay.getIsSingle() || !taMatchWay.getIsSingle()) {
                    throw new Exception("投注比赛明细未开启单关玩法");
                }
            } else {
                if (null == taMatchWay.getIsPass() || !taMatchWay.getIsPass()) {
                    throw new Exception("投注比赛明细未开启过关玩法");
                }
            }
        }

    }

    private void validateDigit(TaBettingPlan taBettingPlan,
                               TaBettingPlanItem taBettingPlanItem,
                               TdLottery tdLottery) throws Exception {
        if (null == taBettingPlanItem.getAmount() || 0 == taBettingPlanItem.getAmount()) {
            throw new Exception("投注注数不能为空");
        }

        Integer charges = taBettingPlanItem.getAmount() * tdLottery.getUnitPrice();
        if (null == taBettingPlanItem.getCharges() || 0 == taBettingPlanItem.getCharges()) {
            taBettingPlanItem.setCharges(charges);
        }

        if (!charges.equals(taBettingPlanItem.getCharges())) {
            throw new Exception("投注金额不正确");
        }

        return;
    }
}
